<?php

namespace Drupal\prometheus_metrics\Bridge;

use Drupal\Core\Site\Settings;
use Prometheus\CollectorRegistry;
use Prometheus\RenderTextFormat;
use Prometheus\Storage\Adapter;
use Prometheus\Storage\APC;
use Prometheus\Storage\InMemory;
use Prometheus\Storage\Redis;

class ArtprimaPHPClientBridge implements PrometheusMetricsInterface
{
  /**
   * @var Adapter
   */
  private $storageAdapter;

  /**
   * @var CollectorRegistry
   */
  private $collectorRegistry;

  /**
   * @var RenderTextFormat
   */
  private $renderer;

  public function __construct(){
    $storageType = Settings::get('prometheus_metrics_storage_type');
    switch($storageType){
      case 'redis':
        $options = [
          'host'=> Settings::get('prometheus_metrics_redis_host'),
          'port'=> Settings::get('prometheus_metrics_redis_port'),
          'timeout' => Settings::get('prometheus_metrics_redis_timeout'),
          'read_timeout' => Settings::get('prometheus_metrics_redis_read_timeout'),
          'persistent_connections' => Settings::get('prometheus_metrics_redis_persist_conns'),
          'password'=>null,
          'database' => Settings::get('prometheus_metrics_redis_database')
        ];
        $this->storageAdapter = new Redis($options);
        break;
      case 'apc':
        $this->storageAdapter = new APC();
        break;
      default:
        //default to in memory storage adapter
        $this->storageAdapter = new InMemory();
        break;
    }

    $this->collectorRegistry = new CollectorRegistry($this->storageAdapter);
    $this->renderer = new RenderTextFormat();
  }

  /**
   * @return RenderTextFormat
   */
  public function getRenderer(): RenderTextFormat
  {
    return $this->renderer;
  }

  /**
   * @return CollectorRegistry
   */
  private function getCollectorRegistry(): CollectorRegistry
  {
    return $this->collectorRegistry;
  }

  public function render(): string
  {
    return $this->getRenderer()
      ->render(
        $this->getCollectorRegistry()->getMetricFamilySamples()
      );
  }

  public function getCounter(string $namespace, string $name, string $help, array $labels)
  {
    return $this->getCollectorRegistry()->getOrRegisterCounter(
      $namespace,
      $name,
      $help,
      $labels
    );
  }

  public function getHistogram(string $namespace, string $name, string $help, array $labels)
  {
    return $this->getCollectorRegistry()->getOrRegisterHistogram(
      $namespace,
      $name,
      $help,
      $labels
    );
  }
}
