<?php

namespace Drupal\prometheus_metrics\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\prometheus_metrics\EventSubscriber\PrometheusDefaults;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Definition of prometheus_metrics route configration form
 */
class MetricsConfigurationForm extends ConfigFormBase
{
  /**
   * @var RouteBuilderInterface
   */
  private $routeBuilder;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param RouteBuilderInterface $routeBuilder
   */
  public function __construct(ConfigFactoryInterface $config_factory, RouteBuilderInterface $routeBuilder) {
    parent::__construct($config_factory);
    $this->routeBuilder = $routeBuilder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('router.builder')
    );
  }

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames()
  {
    return [PrometheusDefaults::CONFIGURATION_NAME];
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId()
  {
    return 'prometheus_metrics_route_configuration_form';
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state)
  {
    $config = $this->config(PrometheusDefaults::CONFIGURATION_NAME);
    $form['metrics_route_path'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Metrics path'),
      '#description' => $this->t('Path to be used by Prometheus to collect metrics. Include preceding slash e.g. /manage/prometheus'),
      '#default_value' => $config->get('metrics_route_path')
    );

    $form['metrics_namespace'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Metrics namespace'),
      '#description' => $this->t('Namespace used for prometheus metrics. Should be unique to this service'),
      '#default_value' => $config->get('metrics_namespace')
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state)
  {
    $path = $form_state->getValue('metrics_route_path');

    if(!UrlHelper::isValid($path)){
      $form_state->setErrorByName('metrics_route_path',$this->t('Path provided is invalid'));
    }
    parent::validateForm($form, $form_state); // TODO: Change the autogenerated stub
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $newRoute = $form_state->getValue('metrics_route_path');
    $oldRoute = $this->config(PrometheusDefaults::CONFIGURATION_NAME)
    ->get('metrics_route_path');

    $this->config(PrometheusDefaults::CONFIGURATION_NAME)
      ->set('metrics_namespace', $form_state->getValue('metrics_namespace'))
      ->save();

    if($newRoute != $oldRoute){
      $this->config(PrometheusDefaults::CONFIGURATION_NAME)
      ->set('metrics_route_path', $newRoute)
      ->save();
      $this->routeBuilder->rebuild();
    }
    parent::submitForm($form, $form_state);
  }
}
