<?php

namespace Drupal\prometheus_metrics\Routing;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\prometheus_metrics\EventSubscriber\PrometheusDefaults;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Defines dynamic route to access prometheus metrics
 */
class MetricsRoutes implements ContainerInjectionInterface
{
  /**
   * @var ConfigFactoryInterface
   */
  private $configFactory;

  /**
   * @var string
   */
  private $metricsRoutePath;

  /**
   * MetricsRoutes constructor.
   * @param ConfigFactoryInterface $configFactory
   */
  public function __construct(ConfigFactoryInterface $configFactory)
  {
    $this->configFactory = $configFactory;
    $config = $this->configFactory->get(PrometheusDefaults::CONFIGURATION_NAME);
    $this->setMetricsRoutePath($config);
  }

  /**
   * Extracts route path from config
   * or defaults to /metrics
   *
   * @param Config $config
   */
  private function setMetricsRoutePath(Config $config):void {
    $this->metricsRoutePath = $config->get('metrics_route_path');
    if(empty($this->metricsRoutePath)) {
      $this->metricsRoutePath = '/metrics';
    }
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container)
  {
    // TODO: Implement create() method.
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function routes() {
    $route_collection = new RouteCollection();

    $route = new Route(
    // Path to attach this route to:
      $this->metricsRoutePath,
      // Route defaults:
      [
        '_controller' => '\Drupal\prometheus_metrics\Controller\MetricsController::collect',
        '_title' => 'Collect metrics'
      ],
      // Route requirements:
      [
        '_permission'  => 'access content',
      ],
      [],
      '',
      [],
      // Methods:
      [
        'GET'
      ]
    );

    $route_collection->add('prometheus_metrics.collect', $route);

    // Returns a route collection object.
    return $route_collection;
  }

}
